/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.ServiceNotFoundException;

import org.ow2.petals.jmx.exception.AttributeErrorException;
import org.ow2.petals.jmx.exception.PerformActionDoesNotExistException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.PerformActionRemoteErrorException;

/**
 * The Petals JMX Client API.
 * @author Christophe Deneux - Capgemini Sud
 */
public abstract class AbstractServiceClient {

    protected final static String JMX_FIELD_TYPE = "type";

    protected final static String JMX_FIELD_NAME = "name";

    protected final static String JMX_SERVICE_TYPE = "service";

    protected String petalsDomain = null;

    protected ObjectName mbeanName = null;

    protected MBeanServerConnection mBeanServerConnection = null;

    /**
     * Creates a new instance of {@link AbstractServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     */
    public AbstractServiceClient(String petalsDomain,
            MBeanServerConnection mBeanServerConnection) {
        this.petalsDomain = petalsDomain;
        this.mBeanServerConnection = mBeanServerConnection;
    }

    /**
     * Perform an action on the MBean service
     * 
     * @param action
     * @return
     * @throws PerformActionDoesNotExistException
     *             The operation invoked does not exist on the server side
     * @throws PerformActionRemoteErrorException
     *             An exception occurs during the remote processing
     * @throws PerformActionErrorException
     *             An exception occurs during the local processing
     */
    protected Object performAction(final String action, final Object[] objectsLoad,
            final String[] stringsLoad)
            throws PerformActionDoesNotExistException, PerformActionRemoteErrorException,
            PerformActionErrorException {
        try {
            return this.mBeanServerConnection.invoke(this.mbeanName, action, objectsLoad,
                        stringsLoad);
        } catch (final MBeanException e) {
            if (e.getCause() instanceof ServiceNotFoundException) {
                // The requested action does not exist on the server MBean side
                throw new PerformActionDoesNotExistException(action, this.mbeanName, e);
            } else {
                // bypass the MBean exception as it doesn't contains any
                // relevant information
                throw new PerformActionRemoteErrorException(e.getCause());
            }
        } catch (final Exception e) {
            throw new PerformActionErrorException(e);
        }
    }

    protected Object getAttribute(String attributeName) throws AttributeErrorException {
        try {
            return this.mBeanServerConnection.getAttribute(this.mbeanName, attributeName);

        } catch (AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (IOException e) {
            throw new AttributeErrorException(e);
        }
    }

    protected Object getAttribute(ObjectName mbeanName, String attributeName)
            throws AttributeErrorException {
        try {
            return this.mBeanServerConnection.getAttribute(mbeanName, attributeName);

        } catch (AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (IOException e) {
            throw new AttributeErrorException(e);
        }
    }

}
