/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.exception.ComponentErrorException;
import org.ow2.petals.jmx.exception.IllegalComponentStateException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;

/**
 * Client API of a JMX component associated to a JBI component.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class ComponentClient extends AbstractServiceClient {

    /**
     * Component states
     */
    // TODO: To share with the Petals kernel side
    public enum State {

        STARTED("Started"), STOPPED("Stopped"), SHUTDOWN("Shutdown"), UNKNOWN("Unknown");

        private final String label;

        private State(final String label) {
            this.label = label;
        }

        public final static State parse(final String label) throws IllegalComponentStateException {
            for (final State state : State.values()) {
                if (state.label.equals(label)) {
                    return state;
                }
            }
            // No existing state matches the value
            throw new IllegalComponentStateException(label);
        }

        public final String toString() {
            return this.label;
        }

    }

    /**
     * Action : shutdown a component
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start a component
     */
    protected static final String START = "start";

    /**
     * Action : stop a component
     */
    protected static final String STOP = "stop";

    /**
     * Action : get current state
     */
    protected static final String GET_CURRENT_STATE = "getCurrentState";

    /**
     * Creates a new instance of {@link ComponentClient}
     * 
     * @param petalsDomain
     * @param componentName
     * @param mBeanServerConnection
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     */
    public ComponentClient(String petalsDomain, String componentName,
            MBeanServerConnection mBeanServerConnection) throws ComponentDoesNotExistException,
            ComponentErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            String pattern = "*:type=engine,name=" + componentName;
            ObjectName objName = new ObjectName(pattern);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                pattern = "*:type=binding,name=" + componentName;
                objName = new ObjectName(pattern);
                objNames = this.mBeanServerConnection.queryNames(objName, null);
                if (objNames != null && objNames.size() == 1) {
                    this.mbeanName = (ObjectName) objNames.iterator().next();
                } else {
                    throw new ComponentDoesNotExistException(componentName);
                }
            }

        } catch (MalformedObjectNameException e) {
            throw new ComponentErrorException(e);
        } catch (IOException e) {
            throw new ComponentErrorException(e);
        }
    }

    /**
     * Creates a new instance of {@link ComponentClient}
     * 
     * @param componentObjectName
     * @param mBeanServerConnection
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     */
    public ComponentClient(ObjectName componentObjectName,
            MBeanServerConnection mBeanServerConnection) throws ComponentDoesNotExistException,
            ComponentErrorException {

        super(componentObjectName.getDomain(), mBeanServerConnection);
        this.mbeanName = componentObjectName;
    }

    /**
     * Start the component associated to this JMX component.
     * 
     * @throws ComponentErrorException
     */
    public void start() throws ComponentErrorException {
        try {
            this.performAction(START, null, null);
        } catch (PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }
    }

    /**
     * Stop the component associated to this JMX component.
     * 
     * @throws ComponentErrorException
     */
    public void stop() throws ComponentErrorException {
        try {
            this.performAction(STOP, null, null);
        } catch (PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }
    }

    /**
     * Shutdown the component associated to this JMX component.
     * 
     * @throws ComponentErrorException
     */
    public void shutdown() throws ComponentErrorException {
        try {
            this.performAction(SHUTDOWN, null, null);
        } catch (PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        }

    }

    /**
     * Get the current state of the JBI component associated to this JMX
     * component.
     * 
     * @return the current state of the JBI component.
     * @throws ComponentErrorException
     */
    public State getCurrentState() throws ComponentErrorException {

        try {
            final String state = (String) this.performAction(GET_CURRENT_STATE, null, null);
            return State.parse(state);
        } catch (final PerformActionErrorException e) {
            throw new ComponentErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentErrorException("Unexpected result type");
        }
    }

}
