/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.exception.EndpointRegistryServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.EndpointRegistryServiceErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.PetalsAdminServiceErrorException;

/**
 * Client API of the JMX Endpoint service.
 * 
 * @author Maxime Carpentier - EBM WebSourcing
 */
public class EndpointRegistryClient extends AbstractServiceClient {

    /**
     * Action : get all internal endpoints
     */
    protected static final String GET_ALL_INTERNAL_ENDPOINTS = "getAllInternalEndpoints";

    /**
     * Action : get all external endpoints
     */
    protected static final String GET_ALL_EXTERNAL_ENDPOINTS = "getAllExternalEndpoints";

    /**
     * Action : get all endpoints
     */
    protected static final String GET_ALL_ENDPOINTS = "getAllEndpoints";

    /**
     * Action : get the description of a given endpoint and service
     */
    protected static final String GET_ENDPOINT_DESCRIPTION = "getDescription";

    /**
     * Action : synchronize registry
     */
    public static final String SYNCHONIZE_REGISTRY = "synchronizeRegistry";

    /**
     * JMX name
     */
    private static final String ENDPOINT_SERVICE_JMX_NAME = "EndpointRegistry";

    /**
     * Creates a new instance of {@link EndpointRegistryServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     */
    public EndpointRegistryClient(final String petalsDomain,
            final MBeanServerConnection beanServerConnection)
            throws EndpointRegistryServiceDoesNotExistException,
            EndpointRegistryServiceErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, ENDPOINT_SERVICE_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new EndpointRegistryServiceDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new EndpointRegistryServiceErrorException(e);
        } catch (IOException e) {
            throw new EndpointRegistryServiceErrorException(e);
        }
    }

    /**
     * Get all the internal Endpoints
     * 
     * @return a list of all internal enpoints. See {@link ReportKeyNames } for
     *         map constants.
     * @throws EndpointRegistryServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<Map<String, Object>> getAllInternalEndpoints()
            throws EndpointRegistryServiceErrorException {
        List<Map<String, Object>> endpoints;

        try {
            endpoints = (List<Map<String, Object>>) this.performAction(GET_ALL_INTERNAL_ENDPOINTS,
                    null, null);
        } catch (PerformActionErrorException e) {
            throw new EndpointRegistryServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new EndpointRegistryServiceErrorException("Unexpected result type");
        }

        return endpoints;
    }

    /**
     * Get all the external Endpoints
     * 
     * @return a list of all external enpoints. See {@link ReportKeyNames } for
     *         map constants.
     * @throws EndpointRegistryServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<Map<String, Object>> getAllExternalEndpoints()
            throws EndpointRegistryServiceErrorException {
        List<Map<String, Object>> endpoints;

        try {
            endpoints = (List<Map<String, Object>>) this.performAction(GET_ALL_EXTERNAL_ENDPOINTS,
                    null, null);
        } catch (PerformActionErrorException e) {
            throw new EndpointRegistryServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new EndpointRegistryServiceErrorException("Unexpected result type");
        }

        return endpoints;
    }

    /**
     * Get all the Endpoints
     * 
     * @return a list of all enpoints. See {@link ReportKeyNames } for map
     *         constants.
     * @throws EndpointRegistryServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<Map<String, Object>> getAllEndpoints() throws EndpointRegistryServiceErrorException {
        List<Map<String, Object>> endpoints;

        try {
            endpoints = (List<Map<String, Object>>) this.performAction(GET_ALL_ENDPOINTS, null,
                    null);
        } catch (PerformActionErrorException e) {
            throw new EndpointRegistryServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new EndpointRegistryServiceErrorException("Unexpected result type");
        }

        return endpoints;
    }

    /**
     * Get the description for a given service and endpoint
     * 
     * @throws EndpointRegistryServiceErrorException
     */
    public String getDescription(String serviceName, String endpointName)
            throws EndpointRegistryServiceErrorException {
        String document;

        Object[] objects = new Object[] { serviceName, endpointName };
        String[] strings = new String[] { String.class.getName(), String.class.getName() };

        try {
            document = (String) this.performAction(GET_ENDPOINT_DESCRIPTION, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new EndpointRegistryServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new EndpointRegistryServiceErrorException("Unexpected result type");
        }

        return document;
    }
    
    public void synchronizeRegistry() throws PetalsAdminServiceErrorException {
        try {
            this.performAction(SYNCHONIZE_REGISTRY, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }
}
