/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;

/**
 * Client API of the JMX installation service.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class InstallationServiceClient extends AbstractServiceClient {

    /**
     * Action : forceUnloadInstaller
     */
    private static final String FORCE_UNLOAD_INSTALLER = "forceUnloadInstaller";

    /**
     * Action : getInstalledComponentsForSharedLibrary
     */
    private static final String GET_COMPONENTS_FOR_SL = "getInstalledComponentsForSharedLibrary";

    /**
     * Action : getInstalledSharedLibraries
     */
    private static final String GET_INSTALLED_SLS = "getInstalledSharedLibraries";

    /**
     * Action : getInstallers
     */
    private static final String GET_INSTALLERS = "getInstallers";

    /**
     * Action : installSharedLibrary
     */
    private static final String INSTALL_SL = "installSharedLibrary";

    /**
     * JMX name
     */
    private final static String INSTALLATION_SERVICE_JMX_NAME = "Installation";

    /**
     * Action : loadInstaller
     */
    private static final String LOAD_INSTALLER = "loadInstaller";

    /**
     * Action : loadNewInstaller
     */
    private static final String LOAD_NEW_INSTALLER = "loadNewInstaller";

    /**
     * Action : shutdownComponent
     */
    private static final String SHUTDOWN_ALL_COMPONENTS = "shutdownAllComponents";

    /**
     * Action : shutdownComponent
     */
    private static final String SHUTDOWN_COMPONENT = "shutdownComponent";

    /**
     * Action : startAllComponent
     */
    private static final String START_ALL_COMPONENTS = "startAllComponents";

    /**
     * Action : startComponent
     */
    private static final String START_COMPONENT = "startComponent";

    /**
     * Action : stopAllComponent
     */
    private static final String STOP_ALL_COMPONENTS = "stopAllComponents";

    /**
     * Action : stopComponent
     */
    private static final String STOP_COMPONENT = "stopComponent";

    /**
     * Action : uninstallAllComponents
     */
    private static final String UNINSTALL_ALL_COMPONENTS = "uninstallAllComponents";

    /**
     * Action : uninstallAllSharedLibrary
     */
    private static final String UNINSTALL_ALL_SL = "uninstallAllSharedLibraries";

    /**
     * Action : uninstallSharedLibrary
     */
    private static final String UNINSTALL_SL = "uninstallSharedLibrary";

    /**
     * Action : unloadAllInstallers.
     */
    private static final String UNLOAD_ALL_INSTALLERS = "unloadAllInstallers";

    /**
     * Action : loadNewInstaller
     */
    private static final String UNLOAD_INSTALLER = "unloadInstaller";

    /**
     * Creates a new instance of {@link InstallationServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     */
    public InstallationServiceClient(String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws InstallationServiceDoesNotExistException, InstallationServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, INSTALLATION_SERVICE_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new InstallationServiceDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new InstallationServiceErrorException(e);
        } catch (IOException e) {
            throw new InstallationServiceErrorException(e);
        }
    }

    /**
     * 
     * Force unload of installer. This method forces the installation of the
     * related component, and then forces the unload of its installer.
     * 
     * @see InstallationServiceMBean#forceUnloadInstaller(String)
     * @param componentName
     * @throws InstallationServiceErrorException
     */
    public boolean forceUnloadInstaller(final String componentName)
            throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(FORCE_UNLOAD_INSTALLER, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Get the components that use a shared library. This API is not defined by
     * JBI.
     * 
     * @return All component identifiers using the provided shared-library.
     * @throws PerformActionErrorException
     */
    public String[] getInstalledComponentsForSharedlibrary(final String sharedLibraryName)
            throws InstallationServiceErrorException {
        String[] slNames;

        Object[] objects = new Object[] { sharedLibraryName };
        String[] strings = new String[] { String.class.getName() };

        try {
            slNames = (String[]) this.performAction(GET_COMPONENTS_FOR_SL, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slNames;
    }

    /**
     * Get all installed shared libraries. This API is not defined by JBI.
     * 
     * @return All installed shared libraries.
     * @throws InstallationServiceErrorException
     */
    public String[] getInstalledSharedLibraries() throws InstallationServiceErrorException {
        String[] slNames;

        try {
            slNames = (String[]) this.performAction(GET_INSTALLED_SLS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slNames;
    }

    /**
     * Get all existing installers. This API is not defined by JBI.
     * 
     * @return All installed shared libraries.
     * @throws InstallationServiceErrorException
     */
    public String[] getInstallers() throws InstallationServiceErrorException {
        final String[] installerNames;

        try {
            installerNames = (String[]) this.performAction(GET_INSTALLERS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return installerNames;
    }

    /**
     * Install a shared-library.
     * 
     * @see InstallationServiceMBean#installSharedLibrary(String)
     * @return The unique name of the shared library installed.
     * @throws InstallationServiceErrorException
     */
    public String installSharedLibrary(final URL sharedLibraryURL)
            throws InstallationServiceErrorException {
        String slName;

        Object[] objects = new Object[] { sharedLibraryURL.toString() };
        String[] strings = new String[] { String.class.getName() };

        try {
            slName = (String) this.performAction(INSTALL_SL, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slName;
    }

    /**
     * Load an installer.
     * 
     * @see InstallationServiceMBean#loadInstaller(String)
     * @param packageURL
     * @return The component installer if it exists. Cannot be <code>null</code>
     *         .
     * @throws InstallerComponentDoesNotExistException
     *             No component installer identified by
     *             <code>componentName</code> was loaded.
     * @throws InstallationServiceErrorException
     */
    public InstallerComponentClient loadInstaller(final String componentName)
            throws InstallerComponentDoesNotExistException, InstallationServiceErrorException {
        ObjectName installerName;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            installerName = (ObjectName) this.performAction(LOAD_INSTALLER, objects, strings);

            if (installerName == null) {
                throw new InstallerComponentDoesNotExistException(componentName);
            } else {
                return new InstallerComponentClient(this.petalsDomain, installerName,
                        this.mBeanServerConnection);
            }

        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }
    }

    /**
     * Load a new installer.
     * 
     * @see InstallationServiceMBean#loadNewInstaller(String)
     * @param packageURL
     * @throws InstallationServiceErrorException
     */
    public InstallerComponentClient loadNewInstaller(final URL packageURL)
            throws InstallationServiceErrorException {
        ObjectName installerName;

        Object[] objects = new Object[] { packageURL.toString() };
        String[] strings = new String[] { String.class.getName() };

        try {
            installerName = (ObjectName) this.performAction(LOAD_NEW_INSTALLER, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return new InstallerComponentClient(this.petalsDomain, installerName,
                this.mBeanServerConnection);
    }

    /**
     * Shutdown all the component.
     * 
     * @return the list of component names that has been shut down
     * @throws InstallationServiceErrorException
     */
    public String[] shutdownAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(SHUTDOWN_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Shutdown the component.
     * 
     * @param componentName
     * @return {@code true} if the shut down was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean shutdownComponent(String componentName) throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(SHUTDOWN_COMPONENT, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Start all the component.
     * 
     * @return the list of component names that has been started
     * @throws InstallationServiceErrorException
     */
    public String[] startAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(START_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Start the component.
     * 
     * @param componentName
     * @return {@code true} if the start was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean startComponent(String componentName) throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(START_COMPONENT, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Stop all the component.
     * 
     * @return the list of component names that has been stopped
     * @throws InstallationServiceErrorException
     */
    public String[] stopAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(STOP_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Stop the component.
     * 
     * @param componentName
     * @return {@code true} if the stop was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean stopComponent(String componentName) throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(STOP_COMPONENT, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Uninstall all the component.
     * 
     * @return the list of component names that has been uninstalled
     * @throws InstallationServiceErrorException
     */
    public String[] uninstallAllComponents() throws InstallationServiceErrorException {
        String[] componentNames;

        try {
            componentNames = (String[]) this.performAction(UNINSTALL_ALL_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * Uninstall all shared-library.
     * 
     * @return the list of the Shared Libraries names that has been uninstalled
     * @throws InstallationServiceErrorException
     */
    public String[] uninstallAllSharedLibrary() throws InstallationServiceErrorException {
        String[] slNames;

        try {
            slNames = (String[]) this.performAction(UNINSTALL_ALL_SL, null, null);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return slNames;
    }

    /**
     * Uninstall a Shared Library.
     * 
     * @see InstallationServiceMBean#uninstallSharedLibrary(String)
     * @return true if the uninstall was successful.
     * @throws InstallationServiceErrorException
     */
    public boolean uninstallSharedLibrary(final String sharedLibraryName)
            throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { sharedLibraryName };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Boolean) this.performAction(UNINSTALL_SL, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }

    /**
     * Unload all the component installers.
     * 
     * @return the list of component names whose installers has been unloaded.
     * @throws InstallationServiceErrorException
     */
    public String[] unloadAllInstallers(boolean force) throws InstallationServiceErrorException {
        String[] componentNames;

        Object[] objects = new Object[] { force };
        String[] strings = new String[] { boolean.class.getName() };

        try {
            componentNames = (String[]) this.performAction(UNLOAD_ALL_INSTALLERS, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return componentNames;
    }

    /**
     * 
     * Unload a component installer. The component is deleted as well.
     * 
     * @see InstallationServiceMBean#unloadInstaller(String, boolean)
     * @param componentName
     * @throws InstallationServiceErrorException
     */
    public boolean unloadInstaller(final String componentName)
            throws InstallationServiceErrorException {
        Boolean result;

        Object[] objects = new Object[] { componentName, true };
        String[] strings = new String[] { String.class.getName(), boolean.class.getName() };

        try {
            result = (Boolean) this.performAction(UNLOAD_INSTALLER, objects, strings);
        } catch (PerformActionErrorException e) {
            throw new InstallationServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new InstallationServiceErrorException("Unexpected result type");
        }

        return result.booleanValue();
    }
}
