/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.exception.ComponentErrorException;
import org.ow2.petals.jmx.exception.ConnectionErrorException;
import org.ow2.petals.jmx.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.exception.EndpointRegistryServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.EndpointRegistryServiceErrorException;
import org.ow2.petals.jmx.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.exception.LoggerDoesNotExistException;
import org.ow2.petals.jmx.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.exception.RouterMonitorServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.RouterMonitorServiceErrorException;
import org.ow2.petals.jmx.exception.RuntimeConfigurationDoesNotExistException;
import org.ow2.petals.jmx.exception.RuntimeConfigurationErrorException;
import org.ow2.petals.jmx.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.monitoring.LocalTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.monitoring.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.monitoring.exception.LocalTransporterMonitoringServiceErrorException;

/**
 * The Petals JMX Client API.
 * @author Christophe Deneux - Capgemini Sud
 * @author Cedric Girodengo - Capgemini Sud
 */
public final class JMXClient {

    private final static String PETALS_DOMAIN = "Petals";

    private final static String OW2_PETALS_DOMAIN = "org.ow2.petals";

    private final static String JAVA_LANG_DOMAIN = "java.lang";

    private JMXConnection jmxConnection = null;

    /**
     * Creates a new instance of {@link JMXClient}
     * 
     * @param host
     * @param jndiPort
     * @param username
     * @param password
     * @throws ConnectionErrorException
     */
    public JMXClient(String host, Integer jndiPort, String username, String password)
            throws ConnectionErrorException {
        this.jmxConnection = new JMXConnection(host, jndiPort, username, password);
    }

    /**
     * Retrieve the Petals Admin service client API
     * 
     * @return the Petals Admin service client API
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public PetalsAdminServiceClient getPetalsAdminServiceClient()
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException,
            ConnectionErrorException {

        return new PetalsAdminServiceClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the Admin service client API
     * 
     * @return the Admin service client API
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     * @throws ConnectionErrorException
     */
    public AdminServiceClient getAdminServiceClient() throws AdminDoesNotExistException,
            AdminServiceErrorException, ConnectionErrorException {

        return new AdminServiceClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the installation service client API
     * 
     * @return the installation service client API
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     * @throws ConnectionErrorException
     */
    public InstallationServiceClient getInstallationServiceClient()
            throws InstallationServiceDoesNotExistException, InstallationServiceErrorException,
            ConnectionErrorException {

        return new InstallationServiceClient(PETALS_DOMAIN, jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the deployment service client API
     * 
     * @return the deployment service client API
     * @throws DeploymentServiceDoesNotExistException
     * @throws DeploymentServiceErrorException
     * @throws ConnectionErrorException
     */
    public DeploymentServiceClient getDeploymentServiceClient()
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException,
            ConnectionErrorException {

        return new DeploymentServiceClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the component client API associated to a JBI component.
     * 
     * @param componentName
     *            Component name
     * @return the component client API
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public ComponentClient getComponentClient(String componentName)
            throws ComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {

        return new ComponentClient(OW2_PETALS_DOMAIN, componentName, jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the component client API associated to a JBI component.
     * 
     * @param componentName
     *            Component name
     * @return the component client API
     * @throws ComponentDoesNotExistException
     * @throws ComponentErrorException
     * @throws ConnectionErrorException
     */
    public ComponentClient getComponentClient(ObjectName componentMBeanName)
            throws ComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {
        if (!componentMBeanName.getDomain().equals(PETALS_DOMAIN)) {
            new ComponentDoesNotExistException("Domain name is not '" + PETALS_DOMAIN + "'");
        }

        return new ComponentClient(componentMBeanName, jmxConnection.getMBeanServerConnection());

    }

    /**
     * Retrieve the component installer client API associated to a JBI
     * component.
     * 
     * @param componentName
     *            Component name
     * @return the installer component client API
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     * @throws ConnectionErrorException
     */
    public InstallerComponentClient getInstallerComponentClient(String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {

        return new InstallerComponentClient(PETALS_DOMAIN, componentName, jmxConnection
                .getMBeanServerConnection());

    }

    /**
     * Retrieve the EndpointRegistry client API
     * 
     * @return the EndpointRegistry component client API
     * @throws ConnectionErrorException
     * @throws EndpointRegistryServiceErrorException
     * @throws EndpointRegistryServiceDoesNotExistException
     */
    public EndpointRegistryClient getEndpointRegistryClient() throws ConnectionErrorException,
            EndpointRegistryServiceDoesNotExistException, EndpointRegistryServiceErrorException {

        return new EndpointRegistryClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());
    }

    /**
     * Retrieve the system monitoring service client API
     * 
     * @return the SystemMonitoring component client API
     * @throws ConnectionErrorException
     * @throws SystemMonitoringServiceErrorException
     * @throws SystemMonitoringDoesNotExistException
     */
    public SystemMonitoringServiceClient getSystemMonitoringServiceClient()
            throws ConnectionErrorException, SystemMonitoringDoesNotExistException,
            SystemMonitoringServiceErrorException {

        return new SystemMonitoringServiceClient(JAVA_LANG_DOMAIN, jmxConnection
                .getMBeanServerConnection());

    }

    public RuntimeConfigurationComponentClient getRuntimeConfigurationClient(
            final String dynamicComponentName) throws RuntimeConfigurationDoesNotExistException,
            RuntimeConfigurationErrorException, ConnectionErrorException {
        return new RuntimeConfigurationComponentClient(OW2_PETALS_DOMAIN, jmxConnection
                .getMBeanServerConnection(), dynamicComponentName);
    }
    
    /**
     * Retrieve the Router Monitor service client API
     * 
     * @return The Router Monitor service client API
     * @throws Router MonitorServiceDoesNotExistException
     * @throws Router MonitorServiceErrorException
     * @throws ConnectionErrorException
     */
    public RouterMonitorServiceClient getRouterMonitoringServiceClient()
            throws RouterMonitorServiceDoesNotExistException, RouterMonitorServiceErrorException,
            ConnectionErrorException {

        return new RouterMonitorServiceClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());
    }
    
    /**
     * Retrieve the Logger service client API
     * @return The Logger service client API
     * @throws LoggerServiceErrorException
     * @throws LoggerDoesNotExistException
     * @throws ConnectionErrorException
     */
    public LoggerServiceClient getLoggerServiceClient() throws LoggerServiceErrorException, LoggerDoesNotExistException, ConnectionErrorException {
        return new LoggerServiceClient(PETALS_DOMAIN, jmxConnection.getMBeanServerConnection());
    }

    /**
     * Retrieve the local transporter monitoring service client API
     * 
     * @return The Logger service client API
     * @throws LocalTransporterMonitoringServiceErrorException
     * @throws LocalTransporterMonitoringServiceDoesNotExistException
     * @throws ConnectionErrorException
     */
    public LocalTransporterMonitoringServiceClient getLocalTransporterMonitoringServiceClient()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        return new LocalTransporterMonitoringServiceClient(PETALS_DOMAIN,
                jmxConnection.getMBeanServerConnection());
    }

    /**
     * Release the connection to the JMX server
     * 
     * @throws ConnectionErrorException
     */
    public void disconnect() throws ConnectionErrorException {
        this.jmxConnection.disconnect();
    }

    /**
     * Get the instance to the @ MBeanServerConnection} in use.
     * 
     * @throws ConnectionErrorException
     */
    public MBeanServerConnection getMBeanServerConnection() throws ConnectionErrorException {
        return this.jmxConnection.getMBeanServerConnection();
    }
}
