/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;
import java.util.logging.Logger;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.LoggerDoesNotExistException;
import org.ow2.petals.jmx.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;

/**
 * Client API of the JMX Logger service.
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class LoggerServiceClient extends AbstractServiceClient {

    /**
     * Action : set level for Logger
     */
    private static final String SET_LEVEL_FOR_LOGGER = "setLevelForLogger";

    /**
     * Action : get level for Loggers
     */
    private static final String GET_LEVEL_FOR_LOGGER = "getLevelForLogger";

    /**
     * Action : get all Levels
     */
    private final static String GET_ALL_LEVELS = "getAllLevels";

    /**
     * Action : get all Loggers
     */
    private final static String GET_ALL_LOGGERS = "getAllLoggers";
    
    private final static String LOGGER_SERVICE_JMX_NAME = "PetalsLogger";

    public LoggerServiceClient(final String petalsDomain,
            final MBeanServerConnection mBeanServerConnection) throws LoggerServiceErrorException,
            LoggerDoesNotExistException {
        super(petalsDomain, mBeanServerConnection);

        try {
            Set<ObjectName> objNames;

            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, LOGGER_SERVICE_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new LoggerDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final IOException e) {
            throw new LoggerServiceErrorException(e);
        }
    }

    /**
     * Set level value for a specified logger
     * 
     * @param logger
     *            The logger name, where the value should be assigned. 
     * @param level
     *            The <String> value of the log level that must be set.
     * @return
     * @throws <LoggerServiceErrorException> if an error occurs while performing
     *         the action
     */
    public void setLevelForLogger(String logger, String level)
            throws LoggerServiceErrorException {

        Object[] objectsLoad = new Object[] { logger, level };
        String[] stringsLoad = new String[] { String.class.getName(), String.class.getName() };
        try {
            this.performAction(SET_LEVEL_FOR_LOGGER, objectsLoad, stringsLoad);
        } catch (final PerformActionErrorException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new LoggerServiceErrorException("Unexpected result type");
        }
    }

    /**
     * Get level value for a specified logger
     * 
     * @param logger
     *            The logger name, where the value should be assigned.
     * @return
     * @throws <LoggerServiceErrorException> if an error occurs while performing
     *         the action
     */
    public String getLevelForLogger(String logger) throws LoggerServiceErrorException {

        Object[] objectsLoad = new Object[] { logger };
        String[] stringsLoad = new String[] { String.class.getName() };

        try {
            return (String) this.performAction(GET_LEVEL_FOR_LOGGER, objectsLoad, stringsLoad);
        } catch (final PerformActionErrorException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new LoggerServiceErrorException("Unexpected result type");
        }

    }

    /**
     * This method allows to retrieve all levels.
     * 
     * @return <String[]> retrieve JDK logging <Level> and specific Petals
     *         logging <Level>.
     */
    public String[] getAllLevels() throws LoggerServiceErrorException {

        try {
            return (String[]) this.performAction(GET_ALL_LEVELS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new LoggerServiceErrorException("Unexpected result type");
        }
    }
    
    /**
     * This method allows to retrieve all loggers.
     * 
     * @return The name (index 0) and level (index 1) of all the {@link Logger}.
     */
    public String[][] getAllLoggers() throws LoggerServiceErrorException {

        try {
            return (String[][]) this.performAction(GET_ALL_LOGGERS, null, null);
        } catch (final PerformActionErrorException e) {
            throw new LoggerServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new LoggerServiceErrorException("Unexpected result type");
        }
    }
}
