/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.exception.PetalsAdminServiceErrorException;

/**
 * Client API of the JMX Petals Admin service.
 * @author Roland Naudin - EBM WebSourcing
 */
public class PetalsAdminServiceClient extends AbstractServiceClient {
    
    /**
     * Action : stop the container
     */
    public static final String STOP = "stopContainer";

    /**
     * Action : shutdown the container
     */
    public static final String SHUTDOWN = "shutdownContainer";

    /**
     * Action : retrieve topology
     */
    public static final String RETRIEVE_TOPOLOGY = "retrieveTopology";

    /**
     * Action : retrieve topology
     */
    public static final String RETRIEVE_SERVER_PROPERTIES = "retrieveServerProperties";
    
    /**
     * JMX name
     */
    private final static String PETALS_ADMIN_JMX_NAME = "PetalsAdmin";

    /**
     * Creates a new instance of {@link PetalsAdminServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @param isPetalsServer20Older
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     */
    public PetalsAdminServiceClient(final String petalsDomain,
            final MBeanServerConnection beanServerConnection)
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, PETALS_ADMIN_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new PetalsAdminDoesNotExistException();
            }
        } catch (IOException e) {
            throw new PetalsAdminServiceErrorException(e);
        } catch (MalformedObjectNameException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * Stop the PEtALS container.
     * 
     * @see PetalsAdminInterface#stopContainer()
     * @throws PerformActionErrorException
     */
    public void stopContainer() throws PetalsAdminServiceErrorException {
        try {
            this.performAction(STOP, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * Shut down the PEtALS container.
     * 
     * @see PetalsAdminInterface#shutdownContainer()
     * @throws PerformActionErrorException
     */
    public void shutdownContainer() throws PetalsAdminServiceErrorException {
        try {
            this.performAction(SHUTDOWN, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    @SuppressWarnings("unchecked")
    public Set<Map<String, String>> retrieveTopology() throws PetalsAdminServiceErrorException {
        final Set<Map<String, String>> result;

        try {
            result = (Set<Map<String, String>>) this.performAction(RETRIEVE_TOPOLOGY, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new PetalsAdminServiceErrorException("Unexpected result type");
        }

        return result;
    }
    
    public Properties retrieveServerProperties() throws PetalsAdminServiceErrorException {
        try {
            return (Properties) this.performAction(RETRIEVE_SERVER_PROPERTIES, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }
}
