/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.RouterMonitorServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.RouterMonitorServiceErrorException;

/**
 * @author Roland Naudin - EBM WebSourcing
 */
public class RouterMonitorServiceClient extends AbstractServiceClient {

    /**
     * JMX name
     */
    private final static String ROUTER_MONITOR_JMX_NAME = "RouterMonitor";

    /**
     * Action : clearMonitorStorage
     */
    private static final String CLEAR_MONITOR_STORAGE = "clearMonitorStorage";

    /**
     * Action : getExchangeHistory
     */
    private static final String GET_EXCHANGE_HISTORY = "getExchangeHistory";

    /**
     * Action : getExchange
     */
    private static final String GET_EXCHANGE = "getExchange";

    /**
     * Action : getExchangeIds
     */
    private static final String GET_EXCHANGE_IDS = "getExchangeIds";

    /**
     * Action : getExchanges
     */
    private static final String GET_EXCHANGES = "getExchanges";

    /**
     * Action : monitorExchanges
     */
    private static final String MONITOR_EXCHANGES = "monitorExchanges";

    /**
     * Action : unmonitorExchanges
     */
    private static final String UNMONITOR_EXCHANGES = "unmonitorExchanges";

    /**
     * Action : getmonitoredExchanges
     */
    private static final String GET_MONITORED_EXCHANGES = "getMonitoredExchanges";

    /**
     * Action : setMonitorStorageDuration
     */
    private static final String SET_MONITOR_STORAGE_DURATION = "setMonitorStorageDuration";

    /**
     * Action : getMonitorStorageDuration
     */
    private static final String GET_MONITOR_STORAGE_DURATION = "getMonitorStorageDuration";

    /**
     * Creates a new instance of {@link RouterMonitorServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     */
    public RouterMonitorServiceClient(String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws RouterMonitorServiceDoesNotExistException, RouterMonitorServiceErrorException {
        super(petalsDomain, mBeanServerConnection);

        try {
            final Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, ROUTER_MONITOR_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            final ObjectName objName = new ObjectName(petalsDomain, attributes);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new RouterMonitorServiceDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (IOException e) {
            throw new RouterMonitorServiceErrorException(e);
        }
    }

    /**
     * Clear the monitor storage.The persisted data related to the monitored
     * exchanges are deleted.
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public void clearMonitorStorage() throws RouterMonitorServiceErrorException {

        try {
            this.performAction(CLEAR_MONITOR_STORAGE, null, null);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        }
    }

    /**
     * 
     * Get the exchange with the given exchange Id.
     * 
     * @param exchangeId
     * @return The exchange in a key/value form
     * 
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public Map<String, String> getExchange(final String exchangeId)
            throws RouterMonitorServiceErrorException {
        final Map<String, String> result;

        final Object[] objects = new Object[] { exchangeId };
        final String[] strings = new String[] { String.class.getName() };

        try {
            result = (Map<String, String>) this.performAction(GET_EXCHANGE, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the history of the given exchange.
     * 
     * @param exchangeId
     *            the send/received history
     * @return The history exchange in a key/value form
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public Map<String, Long> getExchangeHistory(final String exchangeId)
            throws RouterMonitorServiceErrorException {
        final Map<String, Long> result;

        final Object[] objects = new Object[] { exchangeId };
        final String[] strings = new String[] { String.class.getName() };

        try {
            result = (Map<String, Long>) this.performAction(GET_EXCHANGE_HISTORY, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the list of exchange Ids that has been sent or received at least
     * once, matching the given criteria and the exchange persistence duration
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param startDate
     *            The start date
     * @param endDate
     *            The end date
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<String> getExchangeIds(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws RouterMonitorServiceErrorException {
        final List<String> result;

        final Object[] objects = new Object[] { interfaceName, serviceName, endpointName,
                operationName, startDate, endDate };
        final String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName(), long.class.getName(),
                long.class.getName() };

        try {
            result = (List<String>) this.performAction(GET_EXCHANGE_IDS, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the list of exchange Ids that has been sent or received at least
     * once, matching the given criteria and the exchange persistence duration
     * 
     * @param propertyKey
     *            The name of the property
     * 
     * @param propertyValue
     *            The value of the property as a {@code String}
     * 
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<String> getExchangeIds(final String propertyKey, final String propertyValue)
            throws RouterMonitorServiceErrorException {
        final List<String> result;

        final Object[] objects = new Object[] { propertyKey, propertyValue };
        final String[] strings = new String[] { String.class.getName(), String.class.getName() };

        try {
            result = (List<String>) this.performAction(GET_EXCHANGE_IDS, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the number of exchange that has been sent or received at least once
     * and matching the duration
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param startDate
     *            The start date
     * @param endDate
     *            The end date
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    public int getExchanges(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws RouterMonitorServiceErrorException {
        final Integer result;

        final Object[] objects = new Object[] { interfaceName, serviceName, endpointName,
                operationName, startDate, endDate };
        final String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName(), long.class.getName(),
                long.class.getName() };

        try {
            result = (Integer) this.performAction(GET_EXCHANGES, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result.intValue();
    }

    /**
     * Get the list of Router Monitor filter.
     * 
     * @return the list of Router Monitor filter in key/values form.
     * 
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<Map<String, String>> getMonitoredExchange()
            throws RouterMonitorServiceErrorException {
        final List<Map<String, String>> result;

        try {
            result = (List<Map<String, String>>) this.performAction(GET_MONITORED_EXCHANGES, null,
                    null);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the duration of the monitoring storage. Once the duration is reached,
     * the persisted data related to the monitored exchange are deleted.
     * 
     * @return The duration in milliseconds
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public Long getMonitorStorageDuration() throws RouterMonitorServiceErrorException {
        final Long result;

        try {
            result = (Long) (this.performAction(GET_MONITOR_STORAGE_DURATION, null, null));
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Activate the monitoring of the exchanges matching the given criteria.
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param storeExchanges
     *            if {@code true}, the exchanges are stored in the Persistence
     *            Service.
     * @throws RouterMonitorServiceErrorException
     */
    public void monitorExchanges(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final boolean storeExchanges)
            throws RouterMonitorServiceErrorException {

        final Object[] objects = new Object[] { interfaceName, serviceName, endpointName,
                operationName, storeExchanges };
        final String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName(), boolean.class.getName() };

        try {
            this.performAction(MONITOR_EXCHANGES, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        }
    }

    /**
     * Set the duration of the monitoring storage. Once the duration is reached,
     * the persisted data related to the monitored exchange are deleted.
     * 
     * @param duration
     *            The duration in millisecond
     * @throws RouterMonitorServiceErrorException
     */
    public void setMonitorStorageDuration(long duration) throws RouterMonitorServiceErrorException {

        final Object[] objects = new Object[] { duration };
        final String[] strings = new String[] { long.class.getName() };

        try {
            this.performAction(SET_MONITOR_STORAGE_DURATION, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        }
    }

    /**
     * Deactivate the monitoring of the exchanges matching the given criteria.
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param ServiceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public void unmonitorExchanges(QName interfaceName, QName serviceName, String endpointName,
            QName operationName) throws RouterMonitorServiceErrorException {

        final Object[] objects = new Object[] { interfaceName, serviceName, endpointName,
                operationName };
        final String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName() };

        try {
            this.performAction(UNMONITOR_EXCHANGES, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        }
    }

}
