/****************************************************************************
 *
 * Copyright (c) 2009-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.RuntimeConfigurationDoesNotExistException;
import org.ow2.petals.jmx.exception.RuntimeConfigurationErrorException;

/**
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class RuntimeConfigurationComponentClient extends AbstractServiceClient {

    private final static String RUNTIME_CONFIGURATION_JMX_NAME = "runtime_configuration_";

    private final static String RUNTIME_CONFIGURATION_JMX_TYPE = "custom";

    private String componentName = null;

    public RuntimeConfigurationComponentClient(String petalsDomain,
            MBeanServerConnection beanServerConnection, final String componentName)
            throws RuntimeConfigurationDoesNotExistException, RuntimeConfigurationErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            Set<ObjectName> objNames;

            Hashtable<String, String> attributes = new Hashtable<String, String>();
            this.componentName = componentName;

            /*
             * The RUNTIME_CONFIGURATION_JMX_NAME represent the static name part
             * of the focused MBean like "runtime_configuration_", and
             * dynamicComponentName the dynamic name part of this MBean like
             * "petals-bc-filetransfer". finally all the MBean can be:
             * runtime_configuration_petals-bc-filetransfer
             */
            attributes.put(JMX_FIELD_NAME, RUNTIME_CONFIGURATION_JMX_NAME + this.componentName);
            attributes.put(JMX_FIELD_TYPE, RUNTIME_CONFIGURATION_JMX_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new RuntimeConfigurationDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (IOException e) {
            throw new RuntimeConfigurationErrorException(e);
        }
    }

    /**
     * Get Attributes of runtime configuration MBean
     * 
     * @return Map<MBeanAttributeInfo, Object>
     * @throws RuntimeConfigurationErrorException
     */
    public Map<MBeanAttributeInfo, Object> getConfigurationMBeanAttributes()
            throws PerformActionErrorException, RuntimeConfigurationErrorException {
        MBeanAttributeInfo[] attrs = null;
        final Map<MBeanAttributeInfo, Object> attributes = new HashMap<MBeanAttributeInfo, Object>();
        try {
            attrs = this.mBeanServerConnection.getMBeanInfo(this.mbeanName).getAttributes();
            for (final MBeanAttributeInfo info : attrs) {
                attributes.put(info, this.mBeanServerConnection.getAttribute(this.mbeanName, info
                        .getName()));
            }

        } catch (InstanceNotFoundException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (IntrospectionException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (ReflectionException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (IOException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (AttributeNotFoundException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (MBeanException e) {
            throw new RuntimeConfigurationErrorException(e);
        }
        return attributes;
    }

    /**
     * Set Attributes of runtime configuration MBean
     * 
     * @return void
     * @throws RuntimeConfigurationErrorException
     */
    public void setAttributes(final Map<MBeanAttributeInfo, Object> attributes)
            throws PerformActionErrorException, RuntimeConfigurationErrorException {
        final AttributeList attributesList = new AttributeList();
        for (final Entry<MBeanAttributeInfo, Object> entry : attributes.entrySet()) {
            attributesList.add(new Attribute(entry.getKey().getName(), entry.getValue()));
        }
        try {
            this.mBeanServerConnection.setAttributes(this.mbeanName, attributesList);

        } catch (InstanceNotFoundException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (ReflectionException e) {
            throw new RuntimeConfigurationErrorException(e);
        } catch (IOException e) {
            throw new RuntimeConfigurationErrorException(e);
        }
    }

    /**
     * Get the MBean name of this installer.
     * 
     * @return
     */
    public ObjectName getMBeanName() {
        return this.mbeanName;
    }
}
