/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.openmbean.CompositeDataSupport;

import org.ow2.petals.jmx.exception.AttributeErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.exception.SystemMonitoringServiceErrorException;

/**
 * Client API of the JMX System Monitoring service.
 * @author Maxime Carpentier - EBM WebSourcing
 */
public class SystemMonitoringServiceClient extends AbstractServiceClient {

    /**
     * Attributes for ClassLoading MBean
     */
    protected static final String LOADED_CLASS_COUNT = "LoadedClassCount";

    protected static final String TOTAL_LOADED_CLASS_COUNT = "TotalLoadedClassCount";

    protected static final String UNLOADED_CLASS_COUNT = "UnloadedClassCount";

    /**
     * Attributes for Runtime MBean
     */
    protected static final String UP_TIME = "Uptime";

    /**
     * Attributes for Threading MBean
     */
    protected static final String PEAK_THREAD_COUNT = "PeakThreadCount";

    protected static final String DAEMON_THREAD_COUNT = "DaemonThreadCount";

    protected static final String THREAD_COUNT = "ThreadCount";

    protected static final String TOTAL_STARTED_THREAD_COUNT = "TotalStartedThreadCount";

    /**
     * Attributes for Memory MBean
     */
    protected static final String HEAP_MEMORY_USAGE = "HeapMemoryUsage";

    protected static final String OBJECT_PENDING_FINALIZATION_COUNT = "ObjectPendingFinalizationCount";

    /**
     * Attributes for OS MBean : some attributes are platform specific
     */
    protected static final String AVAILABLE_PROCESSORS = "AvailableProcessors";

    protected static final String PROCESS_CPU_TIME = "ProcessCpuTime";

    protected static final String TOTAL_PHYSICAL_MEMORY_SIZE = "TotalPhysicalMemorySize";

    protected static final String FREE_PHYSICAL_MEMORY_SIZE = "FreePhysicalMemorySize";

    protected static final String COMMITTED_VIRTUAL_MEMORY_SIZE = "CommittedVirtualMemorySize";

    /** MBeans of java.lang domain* */
    private final static String OPERATING_SYSTEM = "OperatingSystem";

    private final static String RUNTIME = "Runtime";

    private final static String THREADING = "Threading";

    private final static String MEMORY = "Memory";

    private final static String CLASS_LOADING = "ClassLoading";

    private ObjectName operatingSystemMBeanName;

    private ObjectName runtimeMBeanName;

    private ObjectName threadingMBeanName;

    private ObjectName memoryMBeanName;

    private ObjectName classLoadingMBeanName;

    /**
     * Creates a new instance of {@link SystemMonitoringServiceClient}
     * Check if all needed beans exists
     * 
     * @param domain
     * @param beanServerConnection
     * @throws SystemMonitoringDoesNotExistException if a needed bean does not exists
     */
    public SystemMonitoringServiceClient(String domain, MBeanServerConnection beanServerConnection)
            throws SystemMonitoringDoesNotExistException, SystemMonitoringServiceErrorException {
        super(domain, beanServerConnection);

        try {
            Set<ObjectName> objNames;
            ObjectName objName = new ObjectName(domain + ":*");
            objNames = this.mBeanServerConnection.queryNames(objName, null);

            for (ObjectName bean : objNames) {
                if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + OPERATING_SYSTEM)) {
                    this.operatingSystemMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + RUNTIME)) {
                    this.runtimeMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + THREADING)) {
                    this.threadingMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + MEMORY)) {
                    this.memoryMBeanName = bean;
                } else if (bean.getCanonicalName().equalsIgnoreCase(
                        domain + ":" + JMX_FIELD_TYPE + "=" + CLASS_LOADING)) {
                    this.classLoadingMBeanName = bean;
                }
            }

            if (this.operatingSystemMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + OPERATING_SYSTEM
                        + " bean");
            }
            if (this.runtimeMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + RUNTIME + " bean");
            }
            if (this.threadingMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + THREADING + " bean");
            }
            if (this.memoryMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + MEMORY + " bean");
            }
            if (this.classLoadingMBeanName == null) {
                throw new SystemMonitoringDoesNotExistException("Can't find " + CLASS_LOADING
                        + " bean");
            }

        } catch (MalformedObjectNameException e) {
            throw new SystemMonitoringServiceErrorException(e);
        } catch (IOException e) {
            throw new SystemMonitoringServiceErrorException(e);
        }
    }


    /**
     * Get heap memory usage
     * 
     * @throws PerformActionErrorException
     */
    public CompositeDataSupport getHeapMemoryUsage() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.memoryMBeanName, HEAP_MEMORY_USAGE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof CompositeDataSupport)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((CompositeDataSupport) result);
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }

    }

    /**
     * Get heap memory usage
     * 
     * @throws PerformActionErrorException
     */
    public int getPendingObjects() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.memoryMBeanName, OBJECT_PENDING_FINALIZATION_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }

    }

    /**
     * Get Peak thread count
     * 
     * @throws PerformActionErrorException
     */
    public int getPeakThreadCount() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, PEAK_THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }

    }

    /**
     * Get Daemon thread count
     * 
     * @throws PerformActionErrorException
     */
    public int getDaemonThreadCount() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, DAEMON_THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get thread count
     * 
     * @throws PerformActionErrorException
     */
    public int getThreadCount() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the total started thread count
     * 
     * @throws PerformActionErrorException
     */
    public long getTotalStartedThreadCount() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.threadingMBeanName, TOTAL_STARTED_THREAD_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the available processors
     * 
     * @throws PerformActionErrorException
     */
    public int getAvailableProcessors() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, AVAILABLE_PROCESSORS);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the up time
     * 
     * @throws PerformActionErrorException
     */
    public long getUptime() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.runtimeMBeanName, UP_TIME);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the loaded class count
     * 
     * @throws PerformActionErrorException
     */
    public int getLoadedClassCount() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.classLoadingMBeanName, LOADED_CLASS_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Integer)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Integer) result).intValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the total loaded class count
     * 
     * @throws PerformActionErrorException
     */
    public long getTotalLoadedClassCount() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.classLoadingMBeanName, TOTAL_LOADED_CLASS_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        }catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the unloaded class count
     * 
     * @throws PerformActionErrorException
     */
    public long getUnloadedClassCount() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.classLoadingMBeanName, UNLOADED_CLASS_COUNT);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        }catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the process cpu time
     * 
     * @throws PerformActionErrorException
     * @throws AttributeErrorException
     */
    public long getProcessCpuTime() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, PROCESS_CPU_TIME);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the total physical mem size
     * 
     * @throws PerformActionErrorException
     */
    public long getTotalPhysicalMemorySize() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, TOTAL_PHYSICAL_MEMORY_SIZE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the free physical memory size
     * 
     * @throws PerformActionErrorException
     */
    public long getFreePhysicalMemorySize() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, FREE_PHYSICAL_MEMORY_SIZE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

    /**
     * Get the committed physical memory size
     * 
     * @throws PerformActionErrorException
     */
    public long getCommittedVirtualMemorySize() throws PerformActionErrorException {

        Object result;
        try {
            result = this.getAttribute(this.operatingSystemMBeanName, COMMITTED_VIRTUAL_MEMORY_SIZE);

            if (result == null) {
                throw new PerformActionErrorException("Unexpected result : null");
            } else if (!(result instanceof Long)) {
                throw new PerformActionErrorException("Unexpected result type");
            } else {
                return ((Long) result).longValue();
            }
        } catch (AttributeErrorException e) {
            throw new PerformActionErrorException(e);
        }
    }

}
