/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx.monitoring;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.AbstractServiceClient;
import org.ow2.petals.jmx.exception.AttributeErrorException;
import org.ow2.petals.jmx.monitoring.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.monitoring.exception.LocalTransporterMonitoringServiceErrorException;

/**
 * Client API of the JMX service providing local transporter monitoring metrics.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class LocalTransporterMonitoringServiceClient extends AbstractServiceClient {

    /**
     * Attribute : DelieveredMessagesCount
     */
    private static final String DELIVERED_MESSAGE_COUNT = "DeliveredMessagesCount";

    /**
     * JMX name
     */
    private final static String SERVICE_JMX_NAME = "LocalTransporterMonitoring";

    /**
     * Creates a new instance of {@link LocalTransporterMonitoringServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws LocalTransporterMonitoringServiceDoesNotExistException
     * @throws LocalTransporterMonitoringServiceErrorException
     */
    public LocalTransporterMonitoringServiceClient(final String petalsDomain,
            final MBeanServerConnection mBeanServerConnection)
            throws LocalTransporterMonitoringServiceDoesNotExistException,
            LocalTransporterMonitoringServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, SERVICE_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new LocalTransporterMonitoringServiceDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new LocalTransporterMonitoringServiceErrorException(e);
        } catch (final IOException e) {
            throw new LocalTransporterMonitoringServiceErrorException(e);
        }
    }

    /**
     * Get the metric 'delivered messages'
     * 
     * @throws LocalTransporterMonitoringServiceErrorException
     */
    public Map<String[], Long> getDeliveredMessage()
            throws LocalTransporterMonitoringServiceErrorException {

        try {
            return (Map<String[], Long>) this.getAttribute(DELIVERED_MESSAGE_COUNT);
        } catch (final AttributeErrorException e) {
            throw new LocalTransporterMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new LocalTransporterMonitoringServiceErrorException("Unexpected result type");
        }
    }
}
