/****************************************************************************
 *
 * Copyright (c) 2005-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package org.ow2.petals.jmx;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Hashtable;
import java.util.Set;
import java.util.TreeSet;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Test;
import org.ow2.petals.jmx.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.exception.InstallerComponentDoesNotExistException;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class InstallationServiceClientTest {

    private final static String INEXISTING_COMPONENT_ID = "inexisting-component-id";

    private final static ObjectName INSTALLATION_SERVICE_OBJECT_NAME;

    static {
        final Hashtable<String, String> attributes = new Hashtable<String, String>();
        attributes.put(
                (String) getStaticPrivateField(AbstractServiceClient.class, "JMX_FIELD_NAME"),
                (String) getStaticPrivateField(InstallationServiceClient.class,
                        "INSTALLATION_SERVICE_JMX_NAME"));
        attributes.put(
                (String) getStaticPrivateField(AbstractServiceClient.class, "JMX_FIELD_TYPE"),
                (String) getStaticPrivateField(AbstractServiceClient.class, "JMX_SERVICE_TYPE"));
        try {
            INSTALLATION_SERVICE_OBJECT_NAME = new ObjectName((String) getStaticPrivateField(
                    JMXClient.class, "PETALS_DOMAIN"), attributes);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static Object getStaticPrivateField(final Class<?> clazz, final String fieldName) {
        /* Check we have valid arguments */
        Assert.assertNotNull(clazz);
        Assert.assertNotNull(fieldName);
        /* Go and find the private field... */
        final Field fields[] = clazz.getDeclaredFields();
        for (int i = 0; i < fields.length; ++i) {
            if (fieldName.equals(fields[i].getName())) {
                if (Modifier.isStatic(fields[i].getModifiers())) {
                    try {
                        fields[i].setAccessible(true);
                        return fields[i].get(clazz);
                    } catch (IllegalAccessException ex) {
                        Assert.fail("IllegalAccessException accessing " + fieldName);
                    }
                } else {
                    Assert.fail("Field '" + fieldName + "' not static.");
                }
            }
        }
        Assert.fail("Field '" + fieldName + "' not found");
        return null;
    }

    private static MBeanServerConnection createMBeanServerConnectionMock()
            throws MalformedObjectNameException, IOException {
        final MBeanServerConnection mBeanServerConnectionMock = EasyMock
                .createMock(MBeanServerConnection.class);

        EasyMock.expect(
                mBeanServerConnectionMock.queryNames(INSTALLATION_SERVICE_OBJECT_NAME, null))
                .andAnswer(new IAnswer<Set<ObjectName>>() {

                    @Override
                    public Set<ObjectName> answer() throws Throwable {

                        final Set<ObjectName> objectName = new TreeSet<ObjectName>();
                        objectName.add(INSTALLATION_SERVICE_OBJECT_NAME);
                        return objectName;
                    }
                });

        return mBeanServerConnectionMock;
    }

    /**
     * Check {@link InstallationServiceClient#loadInstaller(String)} against a
     * component loader that was not loaded previously.
     */
    @Test
    public void testLoadInstaller_NotLoaded() throws InstallationServiceDoesNotExistException,
            InstallationServiceErrorException, MalformedObjectNameException, IOException,
            InstanceNotFoundException, MBeanException, ReflectionException {

        final MBeanServerConnection mBeanServerConnectionMock = InstallationServiceClientTest
                .createMBeanServerConnectionMock();

        // Mock the reply of the MBean server about a call to 'loadInstaller'
        // for an inexisting component, according to the JBI specifications.
        Object[] objects = new Object[] { INEXISTING_COMPONENT_ID };
        String[] strings = new String[] { String.class.getName() };
        EasyMock.expect(
                mBeanServerConnectionMock.invoke(EasyMock.eq(INSTALLATION_SERVICE_OBJECT_NAME),
                        EasyMock.eq((String) getStaticPrivateField(InstallationServiceClient.class,
                                "LOAD_INSTALLER")), EasyMock.aryEq(objects), EasyMock
                                .aryEq(strings))).andReturn(null);

        EasyMock.replay(mBeanServerConnectionMock);

        final InstallationServiceClient isc = new InstallationServiceClient("Petals",
                mBeanServerConnectionMock);
        try {
            isc.loadInstaller(INEXISTING_COMPONENT_ID);
            TestCase.fail("The exception '"
                    + InstallerComponentDoesNotExistException.class.getName() + "' is not thrown.");
        } catch (final InstallerComponentDoesNotExistException e) {
            // NOP
            TestCase.assertEquals(INEXISTING_COMPONENT_ID, e.getComponentName());
        }

    }

}
